/*


    ========== licence begin LGPL
    Copyright (C) 2002 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end


*/

package com.sap.dbtech.jdbc.translators;

import java.util.Calendar;
import java.sql.*;
import com.sap.dbtech.util.*;
/**
 *
 */
class TimestampTranslator extends CharDataTranslator {
    public static final long day0 = new Time (0, 0, 0).getTime ();
    public static final long day1 = new Time (23, 59, 59).getTime ();
    public static final String zeroPadC = "0000";
    /**
     * TimeTranslator constructor comment.
     * @param mode int
     * @param ioType int
     * @param len int
     * @param ioLen int
     * @param bufpos int
     */
    protected TimestampTranslator(int mode, int ioType, int dataType, int len, int ioLen, int bufpos) {
        super(mode, ioType, dataType, len, ioLen, bufpos);
    }
    /**
     *
     * @return java.sql.Date
     */
    public Object getObject (
        SQLParamController controller,
        StructuredMem mem) throws SQLException
    {
        return this.getTimestamp (controller, mem);
    }
    /**
     *
     * @return java.sql.Date
     */
    public Date getDate (
        SQLParamController controller,
        StructuredMem mem) throws SQLException
    {
        Timestamp timestamp = this.getTimestamp (controller, mem);
        Date result = null;

        if (timestamp != null) {
            result = new Date (timestamp.getTime ());
        }
        return result;
    }
    /**
     *
     * @return java.sql.Time
     */
    public Time getTime (
        SQLParamController controller,
        StructuredMem mem) throws SQLException
    {
        Timestamp timestamp = this.getTimestamp (controller, mem);
        Time result = null;

        if (timestamp != null) {
            result = new Time (timestamp.getTime ());
        }
        return result;
    }
    /**
     *
     * @return java.sql.Timestamp
     */
    public Timestamp getTimestamp (
        SQLParamController controller,
        StructuredMem mem) throws SQLException
    {
        String raw;
        Timestamp result = null;

        if (!this.isNull (controller, mem)) {
            raw = mem.getString(this.bufpos, this.physicalLength - 1);
            result = Timestamp.valueOf (raw);
        }
        return result;
    }

    public String getString(SQLParamController controller,
                            StructuredMem mem) throws SQLException
     {
         if(this.isNull (controller, mem)) {
             return null;
         }
         String raw;
         raw = mem.getString(this.bufpos, this.physicalLength - 1);
         return raw.substring(0,23);
     }

    /**
     *
     * @return boolean
     */
    public boolean isCaseSensitive () {
        return false;
    }
    /**
     *
     * @param val java.lang.String
     */
    public Object transStringForInput (String val)
    throws SQLException
    {
        if (val == null) {
            return null;
        }
        try {
            return this.transTimestampForInput (Timestamp.valueOf (val));
        }
        catch (IllegalArgumentException exc) {
            // ignore, try timestamp
        }
        try {
            Date d = Date.valueOf (val);
            return this.transTimestampForInput (new Timestamp (d.getTime ()));
        }
        catch (IllegalArgumentException exc) {
            // possibly: try other parsing strategies
        }
        try {
            Time t = Time.valueOf (val);
            return this.transTimestampForInput (new Timestamp (t.getTime ()));
        }
        catch (IllegalArgumentException exc) {
            // possibly: try other parsing strategies
        }
        throw this.newParseException (val, "Timestamp");
    }
    /**
     *
     * @param obj java.lang.Object
     */
    public Object transSpecificForInput (Object obj)
    throws SQLException
    {
        Object result = null;

        if (obj instanceof Timestamp) {
            result = this.transTimestampForInput ((Timestamp) obj);
        }
        else if (obj instanceof Time) {
            String timeString = ((Time) obj).toString ();
            String dateString = new Date (new java.util.Date ().getTime ()).toString ();
            String fullString = dateString + " " + timeString + ".0";
            Timestamp ts = Timestamp.valueOf (fullString);
            result = this.transTimestampForInput (ts);
        }
        else if (obj instanceof Date) {
            String dateString = ((Date) obj).toString ();
            String fullString = dateString + " 00:00:00.0";
            Timestamp ts = Timestamp.valueOf (fullString);
            result = this.transTimestampForInput (ts);
        }
        else if (obj instanceof java.util.Date) {
            java.util.Date utilDate = (java.util.Date) obj;
            result = this.transTimestampForInput (new Timestamp (utilDate.getTime ()));
        }
        else if (obj instanceof String) {
            String utilString = (String) obj;
            result = this.transStringForInput (utilString);
        }
        return result;
    }
    /**
     *
     * @param newValue java.sql.Timestamp
     */
    public Object transTimestampForInput (Timestamp arg)
    throws java.sql.SQLException
    {
        if (arg == null) {
            return null;
        }
        else {
            String result = arg.toString ();
            int lenDiff = 4 - result.indexOf ('-');
            if (lenDiff > 0) {
                result = zeroPadC.substring (0, lenDiff) + result;
            }
            lenDiff = result.length () - this.logicalLength;
            if (lenDiff > 0) {
                result = result.substring (0, this.logicalLength);
            }
            return result;
        }
    }
}
