/*


    ========== licence begin LGPL
    Copyright (C) 2002 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end


*/

package com.sap.dbtech.jdbcext;


import java.sql.*;
import java.util.*;
import java.io.*;
import javax.sql.*;

import com.sap.dbtech.jdbc.*;
import com.sap.dbtech.jdbc.exceptions.*;
import com.sap.dbtech.util.*;

/**
 * This class collects all properties and flags that may be set on 
 * a SAP DB data source.
 */
public class DataSourceSapDBBase
{
    public static final int DEFAULT_LOGINTIMEOUT=30; 

    protected Properties    connectProperties;
    protected PrintWriter   logWriter;
    
    public DataSourceSapDBBase()
    {
        connectProperties=new Properties();
        connectProperties.setProperty("serverName", "localhost");
    }

    public String getUser()
    {
        return connectProperties.getProperty("user");
    }

    public void setLoginTimeout(int timeout)
    {
        connectProperties.setProperty("logintimeout",Integer.toString(timeout));
    }

    public int getLoginTimeout()
    {
        String s=connectProperties.getProperty("logintimeout");
        if(s == null) {
            return DEFAULT_LOGINTIMEOUT;
        }
        try {
            return Integer.parseInt(s);
        } catch(NumberFormatException numberFormatEx) {
            return DEFAULT_LOGINTIMEOUT;
        }            
    }

    public String getPassword()
    {
        return connectProperties.getProperty("password");
    }
    
    public String getDescription() 
    {
        return connectProperties.getProperty("description");
    }
    
    public String getDatabaseName()
    {
        return connectProperties.getProperty("databaseName");
    }

    public String getServerName()
    {
        return connectProperties.getProperty("serverName");
    }

    public String getSqlMode()
    {
        return connectProperties.getProperty("sqlmode", "ADABAS");
    }
    
    public void setUser(String user)
    {
        connectProperties.setProperty("user", user);
    }

    public void setPassword(String password)
    {
        connectProperties.setProperty("password", password);
    }

    public void setDatabaseName(String databaseName)
    {
        connectProperties.setProperty("databaseName", databaseName.toUpperCase());
    }

    public void setServerName(String serverName)
    {
        connectProperties.setProperty("serverName", serverName);
    }
    
    
    public void setUrl(String url)
    {
        setServerName(serverNameFromUrl(url));
        setDatabaseName(databaseNameFromUrl(url));
    }
    
    public String getURL()
    {
        return "jdbc:sapdb://" + getServerName() + "/" + getDatabaseName();
    }

    private String serverNameFromUrl(String url)
    {
       int slashPos;
       int closingSlashPos;
       String host;
       String dbname;
       String dburl;
       int urlLength = url.length ();

       slashPos = "jdbc:sapdb:".length ();
       if ((urlLength >= slashPos + 3)
           && url.substring (slashPos, slashPos + 2).equals ("//")) {
           closingSlashPos = url.indexOf ('/', slashPos + 2);
           host = url.substring (slashPos + 2, closingSlashPos);
           if(host.length()==0) {
               host="localhost";
           }
       }
       else {
           host = "localhost";
       }
       return host;
    }

    private String databaseNameFromUrl(String url)
    {
        int slashPos;
        int closingSlashPos;
        String host;
        String dbname;
        String dburl;
        int urlLength = url.length ();
        
        slashPos = "jdbc:sapdb:".length ();
        if ((urlLength >= slashPos + 3)
            && url.substring (slashPos, slashPos + 2).equals ("//")) {
            closingSlashPos = url.indexOf ('/', slashPos + 2);
            dbname = url.substring(closingSlashPos + 1, urlLength);
        }
        else {
            dbname = url.substring (slashPos, urlLength);
        }
        return dbname.toUpperCase();
    }
    
    public void setSqlMode(String sqlMode) 
    {
        connectProperties.setProperty("sqlmode", sqlMode);
    }
    
    public PrintWriter getLogWriter()
    {
        return this.logWriter;
    }

    public void setLogWriter(PrintWriter logWriter)
    {
        this.logWriter=logWriter;
    }
    
  
    public void setTransport(String transport)
    {
        connectProperties.setProperty("transport", transport);
    }

    public String getTransport()
    {
        return connectProperties.getProperty("transport", "socket");
    }

    public String getCache()
    {
        return connectProperties.getProperty("cache");
    }

    public void setCache(String cache) 
    {
        connectProperties.setProperty("cache", cache);
    }

    public void setCacheSize(String cacheSize)
    {
        connectProperties.setProperty("cachesize", cacheSize);
    }

    public String getCacheSize()
    {
        return connectProperties.getProperty("cachesize");
    }

    public void setUnicode(String unicode)
    {
        connectProperties.setProperty("unicode", unicode);
    }

    public String getUnicode()
    {
        return connectProperties.getProperty("unicode");
    }

    public void setReconnect(String reconnect)
    {
        connectProperties.setProperty("reconnect", reconnect);
    }

    public String getReconnect()
    {
        return connectProperties.getProperty("reconnect");
    }

    public void setTrace(String trace)
    {
        connectProperties.setProperty("trace", trace);
    }
    
    public String getTrace()
    {
        return connectProperties.getProperty("trace");
    }

    public void setTraceSize(String tracesize)
    {
        connectProperties.setProperty("tracesize", tracesize);
    }
    
    public String getTraceSize()
    {
        return connectProperties.getProperty("tracesize");
    }

    public void setXATrace(String xatrace)
    {
        connectProperties.setProperty("xatrace", xatrace);
    }

    public String getXATrace()
    {
        return connectProperties.getProperty("xatrace");
    }

    public void setProperties(Properties p)
    {
        this.connectProperties=(Properties) p.clone();
        // fixup if url is only given ...
        if(connectProperties.getProperty("url")!=null
           && connectProperties.getProperty("databaseName")==null) {
            this.setUrl(connectProperties.getProperty("url"));
        }
    }
    
    public Connection openPhysicalConnection()
        throws SQLException
    {
        Properties p=(Properties)connectProperties.clone();
        String serverName=p.getProperty("serverName", "localhost");
        String databaseName=p.getProperty("databaseName");
        
        if(databaseName==null) {
            throw new SQLExceptionSapDB(MessageTranslator.translate(MessageKey.ERROR_DATABASE_NOT_SET));
        }
        String url = "jdbc:sapdb://" + serverName + "/" + databaseName;
        return DriverSapDB.singleton().connect(url, p);
    }
    
    public Connection openPhysicalConnection(String username, String password)
        throws SQLException
    {
        Properties p=(Properties)connectProperties.clone();
        p.setProperty("user", username);
        p.setProperty("password", password);
        String serverName=p.getProperty("serverName", "localhost");
        String databaseName=p.getProperty("databaseName");
        
        if(databaseName==null) {
            throw new SQLExceptionSapDB(MessageTranslator.translate(MessageKey.ERROR_DATABASE_NOT_SET));
        }
        String url = "jdbc:sapdb://" + serverName + "/" + databaseName;
        return DriverSapDB.singleton().connect(url, p);
    }

    public Connection openPhysicalConnection(Properties p)
        throws SQLException
    {
        String serverName=p.getProperty("serverName", "localhost");
        String databaseName=p.getProperty("databaseName");
        
        if(databaseName==null) {
            throw new SQLExceptionSapDB(MessageTranslator.translate(MessageKey.ERROR_DATABASE_NOT_SET));
        }
        String url = "jdbc:sapdb://" + serverName + "/" + databaseName;
        return DriverSapDB.singleton().connect(url, p);
    }
    
}
