/*
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.sap.dbtech.util;

import java.math.*;
import java.sql.SQLException;
/**
 *
 */
public abstract class VDNNumber {
    private static final int zeroExponentValue_C = 128;
    private static final int tensComplement_C = 9;
    private static final int numberBytes_C = 20;
    private static final int numberDigits_C = 38;
    private static final String zerostring = "0000000000000000000000000000000000000000000000000000000000000000";
    
    /**
     *
     * @return byte[]
     * @param decimal java.math.BigDecimal
     */
    public static byte [] bigDecimal2number (BigDecimal decimal) {
        return bigDecimal2number (decimal, numberDigits_C);
    }
    /**
     *
     * @return byte[]
     * @param decimal java.math.BigDecimal
     */
    public static byte [] bigDecimal2number (BigDecimal decimal, int validDigits) {
        byte [] number = new byte [numberBytes_C];
        int scale = decimal.scale ();
        String plain = decimal.toString ();
        char [] chars = plain.toCharArray ();
        boolean isNegative;
        int firstDigit;
        int exponent;
        int digitCount = chars.length;

        if (chars [0] == '-') {
            isNegative = true;
            firstDigit = 1;
        }
        else {
            isNegative = false;
            firstDigit = 0;
        }
        while (((chars [firstDigit] == '0') || (chars [firstDigit] == '.'))
                && (firstDigit < digitCount - 1)) {
            ++firstDigit;
        }
        exponent = chars.length - firstDigit - scale;
        digitCount = chars.length - firstDigit;
        if ((digitCount == 1) && (chars [firstDigit] == '0')) {
            number [0] = (byte) zeroExponentValue_C;
            return number;
        }
        if ((exponent > 0) && (scale > 0)) {
            // adjust place taken by decimal point
            --exponent;
            --digitCount;
            System.arraycopy(chars, chars.length - scale, chars, chars.length - scale - 1, scale);
        }
        if ((digitCount > validDigits)) {
            if (chars [firstDigit + validDigits] >= '5') {
                ++chars [firstDigit + validDigits];
            }
            digitCount = validDigits;
        }
        for (int i = firstDigit; i < digitCount + firstDigit; ++i) {
            chars [i] -= '0';
        }
        packDigits (chars, firstDigit, digitCount, isNegative, number);
        if (isNegative) {
            exponent = 64 - exponent;
        }
        else {
            exponent += 192;
        }
        number [0] = (byte) exponent;
        return number;
    }
    /**
     *
     * @return byte[]
     * @param value int
     */
    public static byte [] long2number (long value) {
        boolean isNegative = false;
        int negativeVal = 1;
        byte [] number = new byte [numberBytes_C];
        char [] scratch = new char [numberDigits_C + 1];
        char digit;
        int scratchPos = numberDigits_C - 1;
        int exponent;

        if (value == 0) {
            number [0] = (byte) zeroExponentValue_C;
            return number;
        }
        if (value < 0) {
            negativeVal = -1;
            isNegative = true;
        }
        /*
         * calculate digits
         */
        while (value != 0) {
            digit =  (char) (negativeVal*(value % 10));
            scratch [scratchPos] = digit;
            value /= 10;
            --scratchPos;
        }
        exponent = numberDigits_C - scratchPos - 1;
        ++scratchPos;
        packDigits (scratch, scratchPos, numberDigits_C - scratchPos, isNegative, number);
        if (isNegative) {
            exponent = 64 - exponent;
        }
        else {
            exponent += 192;
        }
        number [0] = (byte) exponent;
        return number;
    }
    /**
     *
     * @return java.math.BigDecimal
     * @param rawNumber byte[]
     */
    public static BigDecimal number2BigDecimal(
        byte[] rawNumber) throws SQLException
    {
        BigDecimal result = null;
        int characteristic;
        int digitCount = (rawNumber.length - 1) * 2;
        int exponent;
        byte[] digits;
        int lastSignificant = 2;
        String numberString;

        try{
          characteristic = rawNumber[0] & 0xff;
          if (characteristic == zeroExponentValue_C) {
              return new BigDecimal (0);
          }
          digits = new byte [digitCount + 2];
          if (characteristic < zeroExponentValue_C) {
              exponent = - (characteristic - 64);
              digits [0] = (byte) '-';
              digits [1] = (byte) '.';
              for (int i = 1; i < rawNumber.length; ++i) {
                  int value;
                  value = ((((char) rawNumber [i]) & 0xff) >> 4);
                  if (value != 0) {
                      lastSignificant = i * 2;
                  }
                  digits [i * 2] = (byte) (tensComplement_C - value + '0');
                  value = (((char)rawNumber [i] ) & 0x0f);
                  if (value != 0) {
                      lastSignificant = i * 2 + 1;
                  }
                  digits [i * 2 + 1] = (byte) (tensComplement_C - value + '0');
              }
              ++digits [lastSignificant];
          }
          else {
              exponent = characteristic - 192;
              digits [0] = (byte) '0';
              digits [1] = (byte) '.';
              for (int i = 1; i < rawNumber.length; ++i) {
                  int value;
                  value = ((((char) rawNumber [i]) & 0xff) >> 4);
                  if (value != 0) {
                      lastSignificant = i * 2;
                  }
                  digits [i * 2] = (byte) (value + '0');
                  value = (((char)rawNumber [i] ) & 0x0f);
                  if (value != 0) {
                      lastSignificant = i * 2 + 1;
                  }
                  digits [i * 2 + 1] = (byte) (value + '0');
              }
          }
          numberString = new String(digits, 0, lastSignificant + 1);
          result = new BigDecimal(numberString);
          result = result.movePointRight(exponent);
          return result;
      }
      catch (Exception ex) {
         throw new com.sap.dbtech.jdbc.exceptions.SQLExceptionSapDB(
         MessageTranslator.translate(MessageKey.ERROR_CONVERSIONVDNnumber,
                                     Tracer.Hex2String(rawNumber)));
      }

    }
    /**
     * @return long
     */
    public static long number2long (
        byte [] rawNumber) throws SQLException
    {
        long result = 0;
        int characteristic;
        int exponent;
        boolean isNegative;
        int numberDigits = rawNumber.length * 2 - 2;

        characteristic = rawNumber[0] & 0xff;
        if (characteristic == zeroExponentValue_C) {
            return 0;
        }
        if (characteristic < zeroExponentValue_C) {
            exponent = - (characteristic - 64);
            if ((exponent < 0) || (exponent > numberDigits)) {
                BigDecimal bigD = number2BigDecimal (rawNumber);
                return bigD.longValue();
            }
            isNegative = true;
            for (int i = 0; i < exponent; ++i) {
                int value = rawNumber [i / 2 + 1];
                if ((i % 2) == 0) {
                    value &= 0xf0;
                    value >>>=  4;
                }
                else {
                    value &= 0x0f;
                }
                result *= 10;
                result += tensComplement_C - value;
            }
            ++result;
        }
        else {
            exponent = characteristic - 192;
            if ((exponent < 0) || (exponent > numberDigits)) {
                BigDecimal bigD = number2BigDecimal (rawNumber);
                return bigD.longValue();
            }
            isNegative = false;
            for (int i = 0; i < exponent; ++i) {
                int value = rawNumber [i / 2 + 1];
                if ((i % 2) == 0) {
                    value &= 0xf0;
                    value >>>= 4;
                }
                else {
                    value &= 0x0f;
                }
                result *= 10;
                result += value;
            }
        }
        if (isNegative) {
            result = -result;
        }
        return result;
    }
    /**
     *
     * @return java.math.BigDecimal
     * @param rawNumber byte[]
     */
    public static BigDecimal number2BigDecimal(byte[] rawNumber, int scale) throws SQLException{
        return number2BigDecimal (rawNumber).setScale (scale);
    }
    /**
     *
     * @return int
     * @param rawBytes byte[]
     */
    public static int number2int (byte [] rawBytes) throws SQLException{
        return (int) number2long(rawBytes);
    }
    /**
     *
     */
    private static void packDigits (
        char [] digits,
        int start,
        int count,
        boolean isNegative,
        byte [] number)
    {
        int lastDigit = start + count - 1;
        byte highNibble;
        byte lowNibble;

        if (isNegative) {
            // 10s complement
            for (int i = start; i < lastDigit; ++i) {
                digits [i] = (char) (9 - digits [i]);
            }
            digits [lastDigit] = (char) (10 - digits [lastDigit]);
            // handle overflow
            int digitPos = lastDigit;
            while (digits [digitPos] == 10) {
                digits [digitPos] = 0;
                ++digits [digitPos - 1];
                --digitPos;
            }
        }
        /*
         * pack digits into bytes
         */
        for (int i = 1; start <= lastDigit; ++i, start += 2) {
            highNibble = (byte) digits [start];
            if ((start + 1) <= lastDigit) {
                lowNibble = (byte) digits [start + 1];
            }
            else {
                lowNibble = 0;
            }
            number [i] = (byte) (highNibble << 4 | lowNibble);
        }
    }

    /**
     * Converts a VDN number directly into a string.
     * @param number The number.
     * @param fixedtype Indicator, whether the number is of a fixed type.
     * @param logicallength The logical length of the number (overall number of digits)
     * @param frac The number of fractional digits.
     */
    public static String number2string(byte[] number, boolean fixedtype, int logicalLength, int frac)
        throws SQLException
    {
        int characteristic;
        
        try {
            characteristic = number[0] & 0xFF;
            if(characteristic == zeroExponentValue_C) {
                return "0";
            }
            char digits[] = new char[logicalLength];
            int exponent;
            int lastsignificant=0;
            boolean isnegative=false;
            if(characteristic < zeroExponentValue_C) {
                isnegative=true;
                exponent = - (characteristic - 64);
                for(int i=0; i<logicalLength; ++i) {
                    int v1;
                    if(i % 2 == 0) {
                        v1 = (number[1 + i/2] & 0xff) >> 4;
                    } else {
                        v1 = (number[1 + i/2] & 0xF);
                    }
                    if(v1 != 0) {
                        lastsignificant=i;
                    }
                    digits[i]= (char) ((9 - v1) + '0');
                }
                ++digits[lastsignificant];
            } else {
                exponent = characteristic - 192;
                for(int i=0; i<logicalLength; ++i) {
                    int v1;
                    if(i % 2 == 0) {
                        v1 = (number[1 + i/2] & 0xFF) >> 4;
                    } else {
                        v1 = (number[1 + i/2] & 0xF);
                    }
                    if(v1 != 0) {
                        lastsignificant=i;
                    }
                    digits[i]= (char)((v1) + '0');
                }
            }
            String sign=(isnegative ? "-" : "");
            String numberstr=new String(digits, 0, lastsignificant+1);
            if(fixedtype) {
                if(exponent > 0) {
                    if(numberstr.length() < logicalLength) {
                        numberstr=numberstr + zerostring.substring(0, logicalLength - numberstr.length());
                    }
                    if(frac!=0) {
                        return sign + numberstr.substring(0, exponent) + "." + numberstr.substring(exponent, exponent + frac);
                    } else {
                        return sign + numberstr.substring(0, exponent);
                    }
                } else {
                    int zeroend=frac - (-exponent) - numberstr.length();
                    if(zeroend <0) zeroend = 0;
                    return sign + "0." +
                        zerostring.substring(0, -exponent) + numberstr + zerostring.substring(0, zeroend);
                }
            } else {
                if(exponent <-3 || exponent >7) {
                    return sign + "0." + numberstr + "E" + exponent;
                } else {
                    switch(exponent) {
                    case -3:
                        return sign + "0.000" + numberstr;
                    case -2:
                        return sign + "0.00" + numberstr;
                    case -1:
                        return sign + "0.0" + numberstr;
                    case 0:
                        return sign + "0." + numberstr;
                    }
                    if(numberstr.length() <= exponent) {
                        return sign + numberstr + zerostring.substring(0, exponent - numberstr.length());
                    } else {
                        return sign + numberstr.substring(0, exponent) + "." + numberstr.substring(exponent);
                    }
                }
            }
        } catch(Exception ex) {
            throw new com.sap.dbtech.jdbc.exceptions.SQLExceptionSapDB
                (MessageTranslator.translate(MessageKey.ERROR_CONVERSIONVDNnumber,
                                             Tracer.Hex2String(number)));
        }
    }

            

    /**
     *
     */
    static public void
    dumpVDN (
        String comment,
        byte [] number)
    {
        StructuredBytes struct = new StructuredBytes (number);
        System.out.println (comment); //#print
        struct.traceOn (System.out);
    }
}
